# Copyright (c) 2014 Baidu.com, Inc. All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client class for VPC.
"""

import copy
import json
import logging
import uuid

from baidubce import bce_base_client
from baidubce.auth import bce_v1_signer
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_methods

from baidubce.utils import required
from baidubce import compat

_logger = logging.getLogger(__name__)


class VpcClient(bce_base_client.BceBaseClient):
    """
    VPC base sdk client
    """

    prefix = b'/v1'

    def __init__(self, config=None):
        bce_base_client.BceBaseClient.__init__(self, config)

    def _merge_config(self, config=None):
        """
        :param config:
        :type config: baidubce.BceClientConfiguration
        :return:
        """
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(self, http_method, path,
                      body=None, headers=None, params=None,
                      config=None, body_parser=None):
        config = self._merge_config(config)
        if body_parser is None:
            body_parser = handler.parse_json
        if headers is None:
            headers = {b'Accept': b'*/*', b'Content-Type': b'application/json;charset=utf-8'}
        return bce_http_client.send_request(
            config, bce_v1_signer.sign, [handler.parse_error, body_parser],
            http_method, VpcClient.prefix + path, body, headers, params)

    @required(name=(bytes, str), cidr=(bytes, str))
    def create_vpc(self, name, cidr, description=None, client_token=None, config=None):
        """
        The name of vpc to be created.

        :param name:
            The name of vpc to be created.
        :type name: string

        :param cidr:
            The CIDR of the vpc.
        :type cidr: string

        :param description:
            The description of the vpc.
        :type description: string

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/vpc'
        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'name': compat.convert_to_string(name),
            'cidr': compat.convert_to_string(cidr)
        }
        if description is not None:
            body['description'] = compat.convert_to_string(description)

        return self._send_request(http_methods.POST, path, body=json.dumps(body), params=params,
                                  config=config)

    @required(marker=(bytes, str), max_Keys=int, is_Default=bool)
    def list_vpcs(self, marker=None, max_Keys=None, isDefault=None, config=None):
        """
        Return a list of vpcs owned by the authenticated user.

        :param marker:
            The optional parameter marker specified in the original request to specify
            where in the results to begin listing.
            Together with the marker, specifies the list result which listing should begin.
            If the marker is not specified, the list result will listing from the first one.
        :type marker: string

        :param max_keys:
            The optional parameter to specifies the max number of list result to return.
            The default value is 1000.
        :type max_keys: int

        :param isDefault:
            The option param demotes whether the vpc is default vpc.
        :type isDefault: boolean

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/vpc'
        params = {}

        if marker is not None:
            params[b'marker'] = marker
        if max_Keys is not None:
            params[b'maxKeys'] = max_Keys
        if isDefault is not None:
            params[b'isDefault'] = isDefault

        return self._send_request(http_methods.GET, path, params=params, config=config)

    @required(vpc_id=(bytes, str))
    def get_vpc(self, vpc_id, config=None):
        """
        Get the detail information of specified vpc.

        :param vpc_id:
            The id of vpc.
        :type vpc_id: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/vpc/%s' % compat.convert_to_bytes(vpc_id)

        return self._send_request(http_methods.GET, path, config=config)

    @required(vpc_id=(bytes, str))
    def delete_vpc(self, vpc_id, client_token=None, config=None):
        """
        Delete the specified vpc owned by the user.All resource in the vpc must be deleted before the vpc itself
        can be deleted.

        :param vpc_id:
            The id of instance.
        :type vpc_id: string

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will
            be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/vpc/%s' % compat.convert_to_bytes(vpc_id)
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        return self._send_request(http_methods.DELETE, path, params=params, config=config)

    @required(vpc_id=(bytes, str), name=(bytes, str))
    def update_vpc(self, vpc_id, name, description=None, client_token=None, config=None):
        """
        Modify the special attribute to new value of the vpc owned by the user.

        :param vpc_id:
            The id of the specified vpc
        :type vpc_id: string

        :param name:
            The name of the specified vpc
        :type name: string

        :param description:
           The description of the vpc.
        :type description: string

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """

        path = b'/vpc/%s' % compat.convert_to_bytes(vpc_id)
        params = {
            b'modifyAttribute': None
        }
        body = {
            'name': compat.convert_to_string(name)
        }

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        if description is not None:
            body['description'] = compat.convert_to_string(description)

        return self._send_request(http_methods.PUT, path, json.dumps(body),
                                  params=params, config=config)


def generate_client_token_by_uuid():
    """
    The default method to generate the random string for client_token
    if the optional parameter client_token is not specified by the user.

    :return:
    :rtype string
    """
    return str(uuid.uuid4())


generate_client_token = generate_client_token_by_uuid



# -*- coding: utf-8 -*-

# Copyright (c) 2014 Baidu.com, Inc. All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client class for DDC.
"""

import copy
import json

from baidubce import utils
from baidubce.auth import bce_v1_signer
from baidubce.bce_base_client import BceBaseClient
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_methods
from baidubce.utils import required


class DdcClient(BceBaseClient):
    """
    Ddc base sdk client
    """
    version = '/v1'
    prefix = '/ddc'

    def __init__(self, config=None):
        """
        :type config: baidubce.BceClientConfiguration
        :param config:
        """
        BceBaseClient.__init__(self, config)
        
    @required(instance_id=(str), db_name=(str), table_name=(str))
    def lazydrop_create_hard_link(self, instance_id=None, db_name=None, table_name=None, config=None):
        """
            Create hard link.

            :param instance_id:
                The id of instance
            :type instance_id: string

            :param db_name:
                The database name
            :type db_name: string

            :param table_name:
                The table name
            :type table_name: string

            :param config:
            :type config: baidubce.BceClientConfiguration

            :return: 

            :rtype baidubce.bce_response.BceResponse
        """
        path = DdcClient.version + DdcClient.prefix\
        + "/instance/" + instance_id + "/database/" + db_name + "/table/link"
        body = {}
        if table_name is not None:
            body['tableName'] = table_name
        
        return self._send_request(http_methods.POST, path, body=json.dumps(body), config=config)
    
    @required(instance_id=(str), db_name=(str), table_name=(str))
    def lazydrop_delete_hard_link(self, instance_id=None, db_name=None, table_name=None, config=None):
        """
            Delete hard link.

            :param instance_id:
                The id of instance
            :type instance_id: string

            :param db_name:
                The database name
            :type db_name: string

            :param table_name:
                The table name
            :type table_name: string

            :param config:
            :type config: baidubce.BceClientConfiguration

            :return: 

            :rtype baidubce.bce_response.BceResponse
        """
        path = DdcClient.version + DdcClient.prefix\
        + "/instance/" + instance_id + "/database/" + db_name + "/table/" + table_name + "/link"
        
        return self._send_request(http_methods.DELETE, path, config=config)
    
    @required(instance_id=(str), log_type=(str), datetime=(str))
    def list_log_by_instance_id(self, instance_id=None, log_type=None, datetime=None, config=None):
        """
            Delete hard link.

            :param instance_id:
                The id of instance
            :type instance_id: string

            :param log_type:
                LogType
            :type log_type: string

            :param datetime:
                Datetime
            :type datetime: string

            :param config:
            :type config: baidubce.BceClientConfiguration

            :return: 

            :rtype baidubce.bce_response.BceResponse
        """
        path = '/v2' + DdcClient.prefix + '/instance/' + instance_id + '/logs'
        params = {}
        if log_type is not None:
            params['logType'] = log_type
            
        if datetime is not None:
            params['datetime'] = datetime
        return self._send_request(http_methods.GET, path, params=params, config=config)
    
    @required(instance_id=(str), log_id=(str))
    def get_log_by_id(self, instance_id=None, log_id=None, download_valid_time_in_sec=None, config=None):
        """
            Delete hard link.

            :param instance_id:
                The id of instance
            :type instance_id: string

            :param log_id:
                LogId
            :type log_id: string

            :param download_valid_time_in_sec:
                downloadValidTimeInSec
            :type download_valid_time_in_sec: Integer

            :param config:
            :type config: baidubce.BceClientConfiguration

            :return: 

            :rtype baidubce.bce_response.BceResponse
        """
        path = DdcClient.version + DdcClient.prefix + "/instance/" + instance_id + "/logs/" + log_id
        params = {}
        if download_valid_time_in_sec is not None:
            params['downloadValidTimeInSec'] = download_valid_time_in_sec
        return self._send_request(http_methods.GET, path, params=params, config=config)
    
    @staticmethod
    def _get_path(prefix=None):
        """
        :type prefix: string
        :param prefix: path prefix
        """
        if prefix is None:
            prefix = DdcClient.prefix
        return utils.append_uri(DdcClient.version, prefix)

    def _merge_config(self, config):
        """

        :type config: baidubce.BceClientConfiguration
        :param config:
        :return:
        """
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(self, http_method, path, body=None, headers=None, params=None,
                      config=None, body_parser=None):
        """

        :param http_method:
        :param path:
        :param body:
        :param headers:
        :param params:

        :type config: baidubce.BceClientConfiguration
        :param config:

        :param body_parser:

        :return: baidubce.BceResponse
        """
        config = self._merge_config(config)
        if body_parser is None:
            body_parser = handler.parse_json

        if headers is None:
            headers = {b'Accept': b'*/*', b'Content-Type': b'application/json;charset=utf-8'}

        return bce_http_client.send_request(config, bce_v1_signer.sign,
                                            [handler.parse_error, body_parser],
                                            http_method, path.encode(), body, headers, params)



#! usr/bin/python
# coding=utf-8

# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.


"""
This module defines some Response classes for BTS
"""
from baidubce.bce_response import BceResponse


class CreateSignatureResponse(BceResponse):
    """
    Create Signature Response
    """

    def __init__(self, bce_response):
        super(CreateSignatureResponse, self).__init__()
        self.signature_id = str(bce_response.signature_id)
        self.status = str(bce_response.status)
        self.metadata = bce_response.metadata


class GetSignatureResponse(BceResponse):
    """
    Get Signature Response
    """

    def __init__(self, bce_response):
        super(GetSignatureResponse, self).__init__()
        self.signature_id = str(bce_response.signature_id)
        self.user_id = str(bce_response.user_id)
        self.content = str(bce_response.content)
        self.country_type = str(bce_response.country_type)
        self.content_type = str(bce_response.content_type)
        self.review = str(bce_response.review)
        self.status = str(bce_response.status)
        self.metadata = bce_response.metadata


class CreateTemplateResponse(BceResponse):
    """
    Create Template Response
    """

    def __init__(self, bce_response):
        super(CreateTemplateResponse, self).__init__()
        self.template_id = str(bce_response.template_id)
        self.status = str(bce_response.status)
        self.metadata = bce_response.metadata


class GetTemplateResponse(BceResponse):
    """
    Get Template Response
    """

    def __init__(self, bce_response):
        super(GetTemplateResponse, self).__init__()
        self.template_id = str(bce_response.template_id)
        self.user_id = str(bce_response.user_id)
        self.name = str(bce_response.name)
        self.content = str(bce_response.content)
        self.sms_type = str(bce_response.sms_type)
        self.description = str(bce_response.description)
        self.review = str(bce_response.review)
        self.status = str(bce_response.status)
        self.country_type = str(bce_response.country_type)
        self.metadata = bce_response.metadata


class QueryQuotaResponse(BceResponse):
    """
    Query Quota Response
    """

    def __init__(self, bce_response):
        super(QueryQuotaResponse, self).__init__()
        self.quota_per_day = bce_response.quota_per_day
        self.quota_per_month = bce_response.quota_per_month
        self.quota_remain_today = bce_response.quota_remain_today
        self.quota_remain_this_month = bce_response.quota_remain_this_month
        self.quota_white_list = bce_response.quota_white_list
        self.rate_limit_per_mobile_per_sign_by_minute = bce_response.rate_limit_per_mobile_per_sign_by_minute
        self.rate_limit_per_mobile_per_sign_by_hour = bce_response.rate_limit_per_mobile_per_sign_by_hour
        self.rate_limit_per_mobile_per_sign_by_day = bce_response.reate_limit_per_mobile_per_sign_by_day
        self.rate_limit_white_list = bce_response.rate_limit_white_list
        self.metadata = bce_response.metadata



# Copyright 2017-2019 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client class for VCR.
"""

import copy
import json
import logging
from builtins import str
from builtins import bytes

from baidubce import compat

from baidubce.auth import bce_v1_signer
from baidubce.bce_base_client import BceBaseClient
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_methods
from baidubce.utils import required

_logger = logging.getLogger(__name__)


class VcrClient(BceBaseClient):
    """
    vcr client
    """

    def __init__(self, config=None):
        BceBaseClient.__init__(self, config)

    @required(source=(bytes, str))
    def put_media(self, source, auth=None, description=None,
                  preset=None, notification=None, config=None):
        """
        Check a media.
        :param source: media source
        :type source: string
        :param auth: media source auth param
        :type auth: string
        :param description: media description
        :type description: string
        :param preset: check preset name
        :type preset: string
        :param notification: notification name
        :type notification: string
        :return: **Http Response**
        """
        body = {
            'source': source
        }
        if auth is not None:
            body['auth'] = auth
        if description is not None:
            body['description'] = description
        if preset is not None:
            body['preset'] = preset
        if notification is not None:
            body['notification'] = notification
        return self._send_request(http_methods.PUT, b'/v1/media',
                                  body=json.dumps(body),
                                  config=config)

    @required(source=(bytes, str))
    def get_media(self, source, config=None):
        """
        :param source: media source
        :type source: string
        :return: **Http Response**
        """
        return self._send_request(http_methods.GET, b'/v1/media',
                                  params={b'source': source},
                                  config=config)

    @required(source=(bytes, str))
    def put_audio(self, source, auth=None, description=None,
                  preset=None, notification=None, config=None):
        """
        Check an audio.
        :param source: audio source
        :type source: string
        :param auth: audio source auth param
        :type auth: string
        :param description: audio description
        :type description: string
        :param preset: check preset name
        :type preset: string
        :param notification: notification name
        :type notification: string
        :return: **Http Response**
        """
        body = {
            'source': source
        }
        if auth is not None:
            body['auth'] = auth
        if description is not None:
            body['description'] = description
        if preset is not None:
            body['preset'] = preset
        if notification is not None:
            body['notification'] = notification
        return self._send_request(http_methods.PUT, b'/v2/audio',
                                  body=json.dumps(body),
                                  config=config)

    @required(source=(bytes, str))
    def get_audio(self, source, config=None):
        """
        :param source: audio source
        :type source: string
        :return: **Http Response**
        """
        return self._send_request(http_methods.GET, b'/v2/audio',
                                  params={b'source': source},
                                  config=config)

    @required(source=(bytes, str))
    def put_image(self, source, preset=None, config=None):
        """
        :param source: image source
        :type source: string
        :param preset: check preset name
        :type preset: string
        :return: **Http Response**
        """
        body = {
            'source': source
        }
        if preset is not None:
            body['preset'] = preset
        return self._send_request(http_methods.PUT, b'/v1/image',
                                  body=json.dumps(body),
                                  config=config)

    @required(source=(bytes, str))
    def put_image_async_check(self, source, preset=None, notification=None, description=None,
                              config=None):
        """
        :param source: image source
        :type source: string
        :param preset: check preset name
        :type preset: string
        :param description: image description
        :type description: string
        :param notification: notification name
        :type notification: string
        :return: **Http Response**
        """
        body = {
            'source': source
        }
        if preset is not None:
            body['preset'] = preset
        if description is not None:
            body['description'] = description
        if notification is not None:
            body['notification'] = notification
        return self._send_request(http_methods.PUT, b'/v2/image',
                                  body=json.dumps(body),
                                  config=config)

    @required(source=(bytes, str))
    def get_image_async_check_result(self, source, preset=None, config=None):
        """
        :param source: image source
        :type source: string
        :param preset: check preset name
        :type preset: string
        :return: **Http Response**
        """
        params = {b'source': source}
        if preset is not None:
            params[b'preset'] = preset
        return self._send_request(http_methods.GET, b'/v2/image',
                                  params=params,
                                  config=config)

    @required(text=(bytes, str))
    def put_text(self, text, preset=None, config=None):
        """
        :param text: text to check
        :type text: string
        :param preset: check preset name
        :type preset: string
        :return: **Http Response**
        """
        body = {
            'text': text
        }
        if preset is not None:
            body['preset'] = preset
        return self._send_request(http_methods.PUT, b'/v1/text',
                                  body=json.dumps(body),
                                  config=config)

    @required(lib=(bytes, str), brief=(bytes, str), image=(bytes, str))
    def add_face_image(self, lib, brief, image, config=None):
        """
        :param lib: private face lib
        :param brief: private face brief
        :param image: private face image url
        :return: **Http Response**
        """
        body = {
            'brief': brief,
            'image': image
        }
        return self._send_request(http_methods.POST,
                                  b'/v1/face/lib/%s' % compat.convert_to_bytes(lib),
                                  body=json.dumps(body),
                                  config=config)

    @required(lib=(bytes, str), brief=(bytes, str))
    def del_face_brief(self, lib, brief, config=None):
        """
        :param lib: private face lib
        :param brief: private face brief
        :return: **Http Response**
        """
        params = {
            b'brief': brief
        }
        return self._send_request(http_methods.DELETE,
                                  b'/v1/face/lib/%s' % compat.convert_to_bytes(lib),
                                  params=params,
                                  config=config)

    @required(lib=(bytes, str), brief=(bytes, str), image=(bytes, str))
    def del_face_image(self, lib, brief, image, config=None):
        """
        :param lib: private face lib
        :param brief: private face brief
        :param image: private face image
        :return: **Http Response**
        """
        params = {
            b'brief': brief,
            b'image': image
        }
        return self._send_request(http_methods.DELETE,
                                  b'/v1/face/lib/%s' % compat.convert_to_bytes(lib),
                                  params=params,
                                  config=config)

    @required(lib=(bytes, str))
    def get_face_lib(self, lib, config=None):
        """
        :param lib: private face lib
        :return: **Http Response**
        """
        return self._send_request(http_methods.GET,
                                  b'/v1/face/lib/%s' % compat.convert_to_bytes(lib),
                                  config=config)

    @required(lib=(bytes, str), brief=(bytes, str))
    def get_face_brief(self, lib, brief, config=None):
        """
        :param lib: private face lib
        :param brief: private face brief
        :return: **Http Response**
        """
        params = {
            b'brief': brief
        }
        return self._send_request(http_methods.GET,
                                  b'/v1/face/lib/%s' % compat.convert_to_bytes(lib),
                                  params=params,
                                  config=config)

    @required(lib=(bytes, str), brief=(bytes, str), image=(bytes, str))
    def add_logo_image(self, lib, brief, image, config=None):
        """
        :param lib: private logo lib
        :param brief: private logo brief
        :param image: private logo image
        :return: **Http Response**
        """
        body = {
            'brief': brief,
            'image': image
        }
        return self._send_request(http_methods.POST,
                                  b'/v1/logo/lib/%s' % compat.convert_to_bytes(lib),
                                  body=json.dumps(body),
                                  config=config)

    @required(lib=(bytes, str), brief=(bytes, str))
    def del_logo_brief(self, lib, brief, config=None):
        """
        :param lib: private logo lib
        :param brief: private logo brief
        :return: **Http Response**
        """
        params = {
            b'brief': brief
        }
        return self._send_request(http_methods.DELETE,
                                  b'/v1/logo/lib/%s' % compat.convert_to_bytes(lib),
                                  params=params,
                                  config=config)

    @required(lib=(bytes, str), image=(bytes, str))
    def del_logo_image(self, lib, image, config=None):
        """
        :param lib: private logo lib
        :param image: private logo image
        :return: **Http Response**
        """
        params = {
            b'image': image
        }
        return self._send_request(http_methods.DELETE,
                                  b'/v1/logo/lib/%s' % compat.convert_to_bytes(lib),
                                  params=params,
                                  config=config)

    @required(lib=(bytes, str))
    def get_logo_lib(self, lib, config=None):
        """
        :param lib: private logo lib
        :return: **Http Response**
        """
        return self._send_request(http_methods.GET,
                                  b'/v1/logo/lib/%s' % compat.convert_to_bytes(lib),
                                  config=config)

    @required(lib=(bytes, str), brief=(bytes, str))
    def get_logo_brief(self, lib, brief, config=None):
        """
        :param lib: private logo lib
        :param brief: private logo brief
        :return: **Http Response**
        """
        params = {
            b'brief': brief
        }
        return self._send_request(http_methods.GET,
                                  b'/v1/logo/lib/%s' % compat.convert_to_bytes(lib),
                                  params=params,
                                  config=config)

    @staticmethod
    def _merge_config(self, config):
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(
            self, http_method, path,
            body=None, headers=None, params=None,
            config=None,
            body_parser=None):
        config = self._merge_config(self, config)
        if body_parser is None:
            body_parser = handler.parse_json

        return bce_http_client.send_request(
            config, bce_v1_signer.sign, [handler.parse_error, body_parser],
            http_method, path, body, headers, params)



# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides authentication functions for bce services.
"""
from __future__ import absolute_import
from builtins import str
from builtins import bytes
import hashlib
import hmac
import logging

from baidubce.http import http_headers
from baidubce import utils
from baidubce import compat


_logger = logging.getLogger(__name__)


def _get_canonical_headers(headers, headers_to_sign=None):
    headers = headers or {}

    if headers_to_sign is None or len(headers_to_sign) == 0:
        headers_to_sign = set([b"host",
                               b"content-md5",
                               b"content-length",
                               b"content-type"])
    result = []
    for k in headers:
        k_lower = k.strip().lower()
        value = utils.convert_to_standard_string(headers[k]).strip()
        if k_lower.startswith(http_headers.BCE_PREFIX) \
                or k_lower in headers_to_sign:
            str_tmp = b"%s:%s" % (utils.normalize_string(k_lower), utils.normalize_string(value))
            result.append(str_tmp)
    result.sort()
    return (b'\n').join(result)


def sign(credentials, http_method, path, headers, params,
         timestamp=0, expiration_in_seconds=1800, headers_to_sign=None):
    """
    Create the authorization
    """

    _logger.debug('Sign params: %s %s %s %s %d %d %s' % (
        http_method, path, headers, params, timestamp, expiration_in_seconds, headers_to_sign))

    headers = headers or {}
    params = params or {}

    sign_key_info = b'bce-auth-v1/%s/%s/%d' % (
        credentials.access_key_id,
        utils.get_canonical_time(timestamp),
        expiration_in_seconds)
    sign_key = hmac.new(
        credentials.secret_access_key,
        sign_key_info,
        hashlib.sha256).hexdigest()

    canonical_uri = path
    canonical_querystring = utils.get_canonical_querystring(params, True)

    canonical_headers = _get_canonical_headers(headers, headers_to_sign)
    string_to_sign = (b'\n').join([
        http_method, canonical_uri, 
        canonical_querystring, canonical_headers
        ])
    sign_result = hmac.new(compat.convert_to_bytes(sign_key), string_to_sign, hashlib.sha256).hexdigest()
    # convert to bytes
    sign_result = compat.convert_to_bytes(sign_result)

    if headers_to_sign:
        result = b'%s/%s/%s' % (sign_key_info, (b';').join(headers_to_sign), sign_result)
    else:
        result = b'%s//%s' % (sign_key_info, sign_result)

    _logger.debug('sign_key=[%s] sign_string=[%d bytes][ %s ]' %
                  (sign_key, len(string_to_sign), string_to_sign))
    _logger.debug('result=%s' % result)
    return result



# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides general http handler functions for processing http responses from bos services.
"""

import json
from baidubce import utils
from baidubce.exception import BceServerError
from baidubce.http import handler
from builtins import str
from builtins import bytes

def parse_copy_object_response(http_response, response):
    """
    response parser for copy object
    """
    TRANSFER_ENCODING = b'transfer-encoding'
    headers_list = {k: v for k, v in http_response.getheaders()}
    if headers_list.get(TRANSFER_ENCODING, b'not exist') == b'chunked':
        body = http_response.read()
        if body:
            d = json.loads(body)
            if b'code' in d:
                http_response.close()
                raise BceServerError(d[b'message'], code=d[b'code'], request_id=d[b'requestId'])
            else:
                response.__dict__.update(
                    json.loads(body, object_hook=utils.dict_to_python_object).__dict__)
                http_response.close()
        else:
            e = BceServerError(http_response.reason, request_id=response.metadata.bce_request_id)
            http_response.close()
            raise e
        return True
    else:
        return handler.parse_json(http_response, response)



"""
This module provides a client class for VPN.
"""

import copy
import json
import logging
import uuid

from baidubce import bce_base_client
from baidubce.auth import bce_v1_signer
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_methods

from baidubce import compat

_logger = logging.getLogger(__name__)


class VpnClient(bce_base_client.BceBaseClient):
    """
    VPN base sdk client
    """

    prefix = b'/v1'
    path = b'/vpn'

    def __init__(self, config=None):
        bce_base_client.BceBaseClient.__init__(self, config)

    def _merge_config(self, config=None):
        """
        :param config:
        :type config: baidubce.BceClientConfiguration
        :return:
        """
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(self, http_method, path,
                      body=None, headers=None, params=None,
                      config=None, body_parser=None):
        config = self._merge_config(config)
        if body_parser is None:
            body_parser = handler.parse_json
        if headers is None:
            headers = {b'Accept': b'*/*', b'Content-Type': b'application/json;charset=utf-8'}
        return bce_http_client.send_request(
            config, bce_v1_signer.sign, [handler.parse_error, body_parser],
            http_method, VpnClient.prefix + path, body, headers, params)

    def list_vpns(self, vpcId, eip=None, marker=None, max_Keys=None, config=None):
        """
        return all vpn about vpc

        :param vpcId:
            vpc id
        :type vpcId:string

        :param eip:
            eip
        :type eip:string

        :param marker:
            The optional parameter marker specified in the original request to specify
            where in the results to begin listing.
            Together with the marker, specifies the list result which listing should begin.
            If the marker is not specified, the list result will listing from the first one.
        :type marker: string

        :param max_Keys:
            The optional parameter to specifies the max number of list result to return.
            The default value is 1000.
        :type max_Keys: int

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        params = {b'vpcId': vpcId}

        if marker is not None:
            params[b'marker'] = marker
        if max_Keys is not None:
            params[b'maxKeys'] = max_Keys
        if eip is not None:
            params[b'eip'] = eip

        return self._send_request(http_methods.GET, VpnClient.path, params=params, config=config)

    def create_vpn(self, vpc_id, vpn_name, billing, client_token=None, description=None, eip=None, config=None):
        """
        The method of vpn to be created.

        :param vpc_id:
            vpc id
        :type vpc_id: str

        :param vpn_name:
            the name of name
        :type vpn_name: str

        :param billing:
           order_configuration
        :type billing:Billing

        :param description:
            The description of the vpn.
        :type description: string

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will be used.
        :type client_token: string

        :param eip:
            bind eip
        :type eip:str

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """

        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {'vpcId': vpc_id,
                'vpnName': vpn_name,
                'billing': {
                    'paymentTiming': billing.payment_timing,
                    'billingMethod': billing.billing_method,
                    'reservation': {
                        'reservationLength': billing.reservation_length,
                        'reservationTimeUnit': billing.reservation_time_unit
                    }
                }}

        if description is not None:
            body['description'] = description
        if eip is not None:
            body['eip'] = eip

        return self._send_request(http_methods.POST, VpnClient.path, body=json.dumps(body), params=params,
                                  config=config)

    def update_vpn(self, vpn_id, vpn_name=None, description=None, client_token=None, config=None):
        """
        The method of vpn to be update.

        :param vpn_id: vpn id
        :type vpn_id: str

        :param vpn_name: vpn name
        :type vpn_name: str

        :param description: the description of vpn
        :type description: str

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """

        path = VpnClient.path + b'/' + compat.convert_to_bytes(vpn_id)

        params = {b'modifyAttribute': None}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {}
        if description is not None:
            body['description'] = description
        if vpn_name is not None:
            body['vpnName'] = vpn_name

        return self._send_request(http_methods.PUT, path, body=json.dumps(body), params=params,
                                  config=config)

    def get_vpn(self, vpn_id, config=None):
        """
        Get the detail information of  vpn.

        :param vpn_id:
            The id of vpn.
        :type vpn_id: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = VpnClient.path + b'/' + compat.convert_to_bytes(vpn_id)

        return self._send_request(http_methods.GET, path, config=config)

    def delete_vpn(self, vpn_id, client_token=None, config=None):
        """
        release VPN

        :param vpn_id:
            The id of instance.
        :type vpn_id: string

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will
            be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = VpnClient.path + b'/' + compat.convert_to_bytes(vpn_id)

        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        return self._send_request(http_methods.DELETE, path, params=params, config=config)

    def bind_eip(self, vpn_id, eip=None, client_token=None, config=None):
        """
        bind eip

        :param vpn_id:
            The id of instance.
        :type vpn_id: string

        :param eip:
            The address of eip.
        :type eip: string

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will
            be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = VpnClient.path + b'/' + compat.convert_to_bytes(vpn_id)

        params = {b'bind': None}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {'eip': eip}

        return self._send_request(http_methods.PUT, path, params=params, body=json.dumps(body), config=config)

    def unbind_eip(self, vpn_id, client_token=None, config=None):
        """
        unbind eip

        :param vpn_id:
            The id of instance.
        :type vpn_id: string

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will
            be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = VpnClient.path + b'/' + compat.convert_to_bytes(vpn_id)

        params = {b'unbind': None}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        return self._send_request(http_methods.PUT, path, params=params, config=config)

    def renew_vpn(self, vpn_id, billing, client_token=None, config=None):
        """
        renew vpn

        :param vpn_id:
            The id of instance.
        :type vpn_id: string

        :param billing:
           order_configuration
        :type billing:Billing

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will
            be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = VpnClient.path + b'/' + compat.convert_to_bytes(vpn_id)

        params = {b'purchaseReserved': None}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {'billing': {
            'paymentTiming': billing.payment_timing,
            'billingMethod': billing.billing_method,
            'reservation': {
                'reservationLength': billing.reservation_length,
                'reservationTimeUnit': billing.reservation_time_unit
            }
        }}

        return self._send_request(http_methods.PUT, path, params=params, body=json.dumps(body), config=config)

    def create_vpn_conn(self, vpn_id, secret_key, local_subnets, remote_ip, remote_subnets, vpn_conn_name,
                        ike_config, ipsec_config, description=None, client_token=None, config=None):

        """
        :param vpn_id:vpn id
        :type vpn_id: string

        :param secret_key:shared  key,  8~17  characters,  english,  numbers  and  symbols  must  exist  at
                                 the  same  time,and  the  symbols  are  limited  to  @#$%^*()_
        :type secret_key: string

        :param local_subnets:local network cidr list
        :type local_subnets: list

        :param remote_ip:peer vpn gateway public network ip
        :type remote_ip: string

        :param remote_subnets:peer network cidr list
        :type remote_subnets: list

        :param vpn_conn_name:vpn tunnel name, uppercase and lowercase letters, numbers and -_/. special
                            characters, must start with a letter, length 1-6
        :type vpn_conn_name: list

        :param ike_config:IKE config
        :type ike_config: IkeConfig

        :param ipsec_config:IPSec config
        :type ipsec_config: IpsecConfig

        :param description:description
        :type description: description

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will
            be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = VpnClient.path + b'/' + compat.convert_to_bytes(vpn_id) + b'/vpnconn'

        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'secretKey': secret_key,
            'localSubnets': local_subnets,
            'remoteIp': remote_ip,
            'remoteSubnets': remote_subnets,
            'vpnConnName': vpn_conn_name,
            'ikeConfig': {
                'ike_version': ike_config.ike_version,
                'ike_mode': ike_config.ike_mode,
                'ike_enc_alg': ike_config.ike_enc_alg,
                'ike_auth_alg': ike_config.ike_auth_alg,
                'ike_pfs': ike_config.ike_pfs,
                'ike_lifeTime': ike_config.ike_lifeTime
            },
            'ipsecConfig': {
                'ipsec_enc_alg': ipsec_config.ipsec_enc_alg,
                'ipsec_auth_alg': ipsec_config.ipsec_auth_alg,
                'ipsec_pfs': ipsec_config.ipsec_pfs,
                'ipsec_lifetime': ipsec_config.ipsec_lifetime
            },
            'description': description,
        }

        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), config=config)

    def update_vpn_conn(self, vpn_conn_id, vpn_id, secret_key, local_subnets, remote_ip, remote_subnets, vpn_conn_name,
                        ike_config, ipsec_config, description=None, client_token=None, config=None):

        """
        :param vpn_conn_id:vpnconn id
        :type vpn_conn_id: string

        :param vpn_id:vpn id
        :type vpn_id: string

        :param secret_key:shared  key,  8~17  characters,  english,  numbers  and  symbols  must  exist  at
                                 the  same  time,and  the  symbols  are  limited  to  @#$%^*()_
        :type secret_key: string

        :param local_subnets:local network cidr list
        :type local_subnets: list

        :param remote_ip:peer vpn gateway public network ip
        :type remote_ip: string

        :param remote_subnets:peer network cidr list
        :type remote_subnets: list

        :param vpn_conn_name:vpn tunnel name, uppercase and lowercase letters, numbers and -_/. special
                            characters, must start with a letter, length 1-6
        :type vpn_conn_name: list

        :param ike_config:IKE config
        :type ike_config: IkeConfig

        :param ipsec_config:IPSec config
        :type ipsec_config: IpsecConfig

        :param description:description
        :type description: description

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will
            be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = VpnClient.path + b'/vpnconn/' + compat.convert_to_bytes(vpn_conn_id)

        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'vpnId': vpn_id,
            'secretKey': secret_key,
            'localSubnets': local_subnets,
            'remoteIp': remote_ip,
            'remoteSubnets': remote_subnets,
            'vpnConnName': vpn_conn_name,
            'ikeConfig': {
                'ike_version': ike_config.ike_version,
                'ike_mode': ike_config.ike_mode,
                'ike_enc_alg': ike_config.ike_enc_alg,
                'ike_auth_alg': ike_config.ike_auth_alg,
                'ike_pfs': ike_config.ike_pfs,
                'ike_lifeTime': ike_config.ike_lifeTime
            },
            'ipsecConfig': {
                'ipsec_enc_alg': ipsec_config.ipsec_enc_alg,
                'ipsec_auth_alg': ipsec_config.ipsec_auth_alg,
                'ipsec_pfs': ipsec_config.ipsec_pfs,
                'ipsec_lifetime': ipsec_config.ipsec_lifetime
            },
            'description': description,
        }

        return self._send_request(http_methods.PUT, path, params=params, body=json.dumps(body), config=config)

    def get_vpn_conn(self, vpn_id, client_token=None, config=None):
        """
        :param vpn_id:vpn id
        :type vpn_id: string

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will
            be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = VpnClient.path + b'/vpnconn/' + compat.convert_to_bytes(vpn_id)

        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        return self._send_request(http_methods.GET, path, params=params, config=config)

    def delete_vpn_conn(self, vpn_conn_id, client_token=None, config=None):
        """
        :param vpn_conn_id:vpn conn id
        :type vpn_conn_id: string

        :param client_token:
            An ASCII string whose length is less than 64.
            The request will be idempotent if clientToken is provided.
            If the clientToken is not specified by the user, a random String generated by default algorithm will
            be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = VpnClient.path + b'/vpnconn/' + compat.convert_to_bytes(vpn_conn_id)

        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        return self._send_request(http_methods.DELETE, path, params=params, config=config)


def generate_client_token_by_uuid():
    """
    The default method to generate the random string for client_token
    if the optional parameter client_token is not specified by the user.

    :return:
    :rtype string
    """
    return str(uuid.uuid4())


generate_client_token = generate_client_token_by_uuid



# Copyright (c) 2019 Baidu.com, Inc. All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client class for APP BLB.
"""

import copy
import json
import logging
import uuid
import sys

from baidubce import bce_base_client
from baidubce.auth import bce_v1_signer
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_methods
from baidubce import utils
from baidubce.utils import required
from baidubce import compat

if sys.version < '3':
    sys.setdefaultencoding('utf-8')

_logger = logging.getLogger(__name__)


class AppBlbClient(bce_base_client.BceBaseClient):
    """
    APP BLB base sdk client
    """
    version = b'/v1'

    def __init__(self, config=None):
        bce_base_client.BceBaseClient.__init__(self, config)

    def _merge_config(self, config=None):
        """
        :param config:
        :type config: baidubce.BceClientConfiguration
        :return:
        """
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(self, http_method, path,
                      body=None, headers=None, params=None,
                      config=None, body_parser=None):
        config = self._merge_config(config)
        if body_parser is None:
            body_parser = handler.parse_json
        if headers is None:
            headers = {b'Accept': b'*/*',
                       b'Content-Type': b'application/json;charset=utf-8'}
        return bce_http_client.send_request(
            config, bce_v1_signer.sign, [handler.parse_error, body_parser],
            http_method, path, body, headers, params)

    @required(vpc_id=(bytes, str),
              subnet_id=(bytes, str))
    def create_app_loadbalancer(self, vpc_id, subnet_id, name=None,
                                desc=None, client_token=None, config=None):
        """
        Create a app LoadBalancer with the specified options.

        :param name:
                the name of LoadBalancer to create
        :type name: string

        :param desc:
                The description of LoadBalancer
        :type desc: string

        :param vpc_id:
                id of vpc which the LoadBalancer belong to
        :type vpc_id: string

        :param subnet_id:
                id of subnet which the LoadBalancer belong to
        :type subnet_id: string

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb')
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {}
        if name is not None:
            body['name'] = compat.convert_to_string(name)
        if desc is not None:
            body['desc'] = compat.convert_to_string(desc)
        body['vpcId'] = compat.convert_to_string(vpc_id)
        body['subnetId'] = compat.convert_to_string(subnet_id)

        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str))
    def update_app_loadbalancer(self, blb_id, name=None, desc=None,
                                client_token=None, config=None):
        """
        Modify the special attribute to new value of the LoadBalancer
        owned by the user.

        :param name:
                name of LoadBalancer to describe
        :type name: string

        :param blb_id:
                id of LoadBalancer to describe
        :type blb_id: string

        :param desc:
                The description of LoadBalancer
        :type desc: string

        :param client_token:
                If the clientToken is not specified by the user,
                a random String generated by default algorithm
                will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id)
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {}
        if name is not None:
            body['name'] = compat.convert_to_string(name)
        if desc is not None:
            body['desc'] = compat.convert_to_string(desc)

        return self._send_request(http_methods.PUT, path, json.dumps(body),
                                  params=params, config=config)

    def describe_app_loadbalancers(self, address=None, name=None, blb_id=None,
                                   bcc_id=None, marker=None, max_keys=None,
                                   config=None):
        """
        Return a list of LoadBalancers

        :param address:
            Intranet service address in dotted decimal notation
        :type address: string

        :param name:
            name of LoadBalancer to describe
        :type name: string

        :param blb_id:
            id of LoadBalancer to describe
        :type blb_id: string

        :param bcc_id:
            bcc which bind the LoadBalancers
        :type bcc_id: string

        :param marker:
            The optional parameter marker specified in the original
            request to specify where in the results to begin listing.
            Together with the marker, specifies the list result
            which listing should begin.
            If the marker is not specified, the list result will
            listing from the first one.
        :type marker: string

        :param max_keys
        The optional parameter to specifies the max number of list
        result to return.
            The default value is 1000.
        :type max_keys: int

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb')
        params = {}

        if address is not None:
            params[b'address'] = address
        if name is not None:
            params[b'name'] = name
        if blb_id is not None:
            params[b'blbId'] = blb_id
        if bcc_id is not None:
            params[b'bccId'] = bcc_id
        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys

        return self._send_request(http_methods.GET, path,
                                  params=params, config=config)

    @required(blb_id=(bytes, str))
    def describe_app_loadbalancer_detail(self, blb_id, config=None):
        """
        Return detail imformation of specific LoadBalancer

        :param blb_id:
            id of LoadBalancer to describe
        :type blb_id: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id)

        return self._send_request(http_methods.GET, path,
                                  config=config)

    @required(blb_id=(bytes, str))
    def delete_app_loadbalancer(self, blb_id, client_token=None, config=None):
        """
        delete the LoadBalancer owned by the user.

        :param blb_id:
                id of LoadBalancer to describe
        :type blb_id: string

        :param client_token:
                If the clientToken is not specified by the user,
                a random String generated by default algorithm
                will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id)
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token
        return self._send_request(http_methods.DELETE, path,
                                  params=params, config=config)

    """
        Listener API
    """

    @required(blb_id=(bytes, str),
              listener_port=int,
              scheduler=(bytes, str))
    def create_app_tcp_listener(self, blb_id, listener_port,
                                scheduler, client_token=None,
                                config=None):
        """
        Create a app tcp listener rule with the specified options.

        :param blb_id:
            the id of blb which the listener work on
        :type blb_id: string

        :param listener_port:
            port to be linstened owned by listener
        :value 1-65535
        :type listener_port: int

        :param scheduler
            balancing algorithm
        :value 'RoundRobin' or 'LeastConnection' or 'Hash'
        :type scheduler: string

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

         :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'TCPlistener')
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'listenerPort': listener_port,
            'scheduler': compat.convert_to_string(scheduler)
        }

        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              listener_port=int,
              scheduler=(bytes, str))
    def create_app_udp_listener(self, blb_id, listener_port,
                                scheduler, client_token=None,
                                config=None):
        """
        Create a app udp listener rule with the specified options.

        :param blb_id:
            the id of blb which the listener work on
        :type blb_id: string

        :param listener_port:
            port to be linstened owned by listener
        :value 1-65535
        :type listener_port: int

        :param scheduler
            balancing algorithm
        :value 'RoundRobin' or 'LeastConnection' or 'Hash'
        :type scheduler: string

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

         :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'UDPlistener')
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'listenerPort': listener_port,
            'scheduler': compat.convert_to_string(scheduler)
        }

        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str), listener_port=int,
              scheduler=(bytes, str))
    def create_app_http_listener(self, blb_id, listener_port,
                                 scheduler, keep_session=None,
                                 keep_session_type=None,
                                 keep_session_timeout=None,
                                 keep_session_cookie_name=None,
                                 x_forward_for=None,
                                 server_timeout=None,
                                 redirect_port=None,
                                 client_token=None,
                                 config=None):
        """
        Create a app http listener rule with the specified options.
        :param blb_id:
            the id of blb which the listener work on
        :type blb_id: string

        :param listener_port:
            port to be linstened owned by listener
        :value 1-65535
        :type listener_port: int

        :param scheduler:
            balancing algorithm
        :value 'RoundRobin' or 'LeastConnection'
        :type scheduler: string

        :param keep_session:
            Whether to enable the session hold function,
            that is,the request sent by the same client will
            reach the same backend server
        :value true or false  default:false
        :type keep_session: bool

        :param keep_session_type:
            The cookie handling method maintained by the session,
            valid only if the session is held open
            :value 'insert' or 'rewrite'  default:insert
        :type keep_session_type: string

        :param keep_session_timeout:
            The time the cookie is kept in session (in seconds),
            valid only if the session is held open
            :value 1-15552000  default:3600
        :type keep_session_timeout: int

        :param keep_session_cookie_name:
            The session keeps the name of the cookie that needs to be
            overridden if and only if session persistence is enabled
            and keep_session_type="rewrite"
        :type keep_session_cookie_name: int

        :param x_forward_for:
            Whether to enable the real IP address of the client,
            the backend server can obtain the real address of the client
            through the X-Forwarded-For HTTP header.
        :value true or false, default: False
        :type x_forward_for: bool

        :param server_timeout:
            Backend server maximum timeout (unit: second)
        :value 1-3600, default: 30
        :type server_timeout:int

        :param redirect_port:
            Forward the request received by this listener to the
            HTTPS listener, which is specified by the HTTPS listener.
        :type redirect_port:int

        :param client_token:
            If the clientToken is not specified by the user,
            a random String generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'HTTPlistener')
        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token
        body = {
            'listenerPort': listener_port,
            'scheduler': compat.convert_to_string(scheduler)}
        if keep_session is not None:
            body['keepSession'] = keep_session
        if keep_session_type is not None:
            body['keepSessionType'] = \
                compat.convert_to_string(keep_session_type)
        if keep_session_timeout is not None:
            body['keepSessionTimeout'] = keep_session_timeout
        if keep_session_cookie_name is not None:
            body['keepSessionCookieName'] = keep_session_cookie_name
        if x_forward_for is not None:
            body['xForwardFor'] = x_forward_for
        if server_timeout is not None:
            body['serverTimeout'] = server_timeout
        if redirect_port is not None:
            body['redirectPort'] = redirect_port
        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str), listener_port=int,
              scheduler=(bytes, str), cert_ids=list)
    def create_app_https_listener(self, blb_id, listener_port,
                                  scheduler, cert_ids,
                                  keep_session=None,
                                  keep_session_type=None,
                                  keep_session_timeout=None,
                                  keep_session_cookie_name=None,
                                  x_forward_for=None, server_timeout=None,
                                  ie6_compatible=None, encryption_type=None,
                                  encryption_protocols=None,
                                  dual_auth=None, client_certIds=None,
                                  client_token=None, config=None):
        """
        Create a app https listener rule with the specified options.
        :param blb_id:
            The id of blb which the listener work on
        :type blb_id: string
        :param listener_port:
            port to be linstened owned by listener
        :value 1-65535
        :type listener_port: int
        :param scheduler:
            balancing algorithm
        :value 'RoundRobin' or 'LeastConnection'
        :type scheduler: string
        :param cert_ids:
            The certificate to be loaded by the listener.
        :type cert_ids: List<String>
        :param keep_session:
            Whether to enable the session hold function,
            that is, the request sent by the same client will reach the
            same backend server
        :value true or false, default: false
        :type keep_session: bool
        :param keep_session_type:
            The cookie handling method maintained by the session,
            valid only if the session is held open
        :value 'insert' or 'rewrite', default:insert
        :type keep_session_type: string
        :param keep_session_timeout:
            The time the cookie is kept in session (in seconds),
            valid only if the session is held open
        :value 1-15552000, default:3600
        :type keep_session_timeout: int
        :param keep_session_cookie_name:
            The session keeps the name of the cookie that needs
            to be overridden if and only if session persistence
            is enabled and keep_session_type="rewrite"
        :type keep_session_cookie_name: int
        :param x_forward_for:
            Whether to enable the real IP address of the client,
            the backend server can obtain the real address of the client
            through the X-Forwarded-For HTTP header.
        :value true or false, default: flase
        :type x_forward_for: bool
        :param server_timeout:
            Backend server maximum timeout (unit: second)
        :value 1-3600, default: 30
        :type server_timeout: int
        :param ie6_compatible:
            compatible with IE6 HTTPS request
            (the protocol format is earlier SSL3.0, the security is poor)
        :value true or false, default: true
        :type ie6_compatible: bool
        :param encryption_type:
            Encryption options, support three types:
            compatibleIE/incompatibleIE/userDefind,
            corresponding to:
            IE-compatible encryption/disabled unsecure encryption/custom encryption,
            when encryptionType is valid and legitimate,
            ie6Compatible field transfer value will not take effect
        type: encryption_type:string
        :param encryption_protocols:
            When the encryptionType value is userDefind,
            the list of protocol types is a string list composed of four protocols:
            "sslv3", "tlsv10", "tlsv11", "tlsv12".
        type: encryption_protocols:list
        :param dual_auth:
            Whether to Open Two-way Authentication,
            default:false
        :type dual_auth: boolean
        :param client_certIds:
            When dualAuth is true, the loaded client certificate chain
        :type client_certIds: list
        :param client_token:
            If the clientToken is not specified by the user,
            a random String generated by default algorithm will be used.
        :type client_token: string
        :param config:
        :type config: baidubce.BceClientConfiguration
        :return
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'HTTPSlistener')
        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token
        body = {
            'listenerPort': listener_port,
            'scheduler': compat.convert_to_string(scheduler),
            'certIds': cert_ids}
        if keep_session is not None:
            body['keepSession'] = keep_session
        if keep_session_type is not None:
            body['keepSessionType'] = \
                compat.convert_to_string(keep_session_type)
        if keep_session_timeout is not None:
            body['keepSessionTimeout'] = keep_session_timeout
        if keep_session_cookie_name is not None:
            body['keepSessionCookieName'] = keep_session_cookie_name
        if x_forward_for is not None:
            body['xForwardFor'] = x_forward_for
        if server_timeout is not None:
            body['serverTimeout'] = server_timeout
        if ie6_compatible is not None:
            body['ie6Compatible'] = ie6_compatible
        if encryption_type is not None:
            body['encryptionType'] = \
                compat.convert_to_string(encryption_type)
        if encryption_protocols is not None:
            body['encryptionProtocols'] = encryption_protocols
        if dual_auth is not None:
            body['dualAuth'] = dual_auth
        if client_certIds is not None:
            body['clientCertIds'] = client_certIds
        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body),
                                  params=params, config=config)

    @required(blb_id=(bytes, str), listener_port=int,
              scheduler=(bytes, str), cert_ids=list)
    def create_app_ssl_listener(self, blb_id, listener_port,
                                scheduler, cert_ids,
                                ie6_compatible=None,
                                encryption_type=None,
                                encryption_protocols=None,
                                dual_auth=None, client_certIds=None,
                                client_token=None, config=None):
        """
        Create a app ssl listener rule with the specified options.

        :param blb_id:
            The id of blb which the listener work on
        :type blb_id: string

        :param listener_port:
            port to be linstened owned by listener
        :value 1-65535
        :type listener_port: int

        :param scheduler:
            balancing algorithm
        :value 'RoundRobin' or 'LeastConnection'
        :type scheduler: string

        :param cert_ids:
            The SSL certificate to be loaded by the listener.
            Currently HTTPS listeners can only bind one SSL certificate.
        :type cert_ids: List<String>

        :param ie6_compatible:
            compatible with IE6 HTTPS request
            (the protocol format is earlier SSL3.0, the security is poor)
        :value true or false, default: true
        :type ie6_compatible: bool

        :param encryption_type:
            Encryption options, support three types:
            compatibleIE/incompatibleIE/userDefind,
            corresponding to:
            IE-compatible encryption/disabled unsecure encryption/custom encryption,
            when encryptionType is valid and legitimate,
            ie6Compatible field transfer value will not take effect
        type: encryption_type:string

        :param encryption_protocols:
            When the encryptionType value is userDefind,
            the list of protocol types is a string list composed of four protocols:
            "sslv3", "tlsv10", "tlsv11", "tlsv12".
        type: encryption_protocols:list

        :param dual_auth:
            Whether to Open Two-way Authentication,
            default:false
        :type dual_auth: boolean

        :param client_certIds:
            When dualAuth is true, the loaded client certificate chain
        :type client_certIds: list

        :param client_token:
            If the clientToken is not specified by the user,
            a random String generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'SSLlistener')
        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token
        body = {
            'listenerPort': listener_port,
            'scheduler': compat.convert_to_string(scheduler),
            'certIds': cert_ids}
        if ie6_compatible is not None:
            body['ie6Compatible'] = ie6_compatible
        if encryption_type is not None:
            body['encryptionType'] = \
                compat.convert_to_string(encryption_type)
        if encryption_protocols is not None:
            body['encryptionProtocols'] = encryption_protocols
        if dual_auth is not None:
            body['dualAuth'] = dual_auth
        if client_certIds is not None:
            body['clientCertIds'] = client_certIds

        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body),
                                  params=params, config=config)

    @required(blb_id=(bytes, str),
              listener_port=int)
    def update_app_tcp_listener(self, blb_id, listener_port,
                                scheduler=None,
                                client_token=None,
                                config=None):
        """
        update a app tcp listener rule with the specified options.

        :param blb_id:
            the id of blb which the listener work on
        :type blb_id:string

        :param listener_port:
             port to be linstened owned by listener
        :value 1-65535
        :type listener_port:int

        :param scheduler
            balancing algorithm
        :value 'RoundRobin'or'LeastConnection'or'Hash'
        :type scheduler:string

        :param client_token:
                If the clientToken is not specified by the user,
                a random String generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'TCPlistener')
        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token
        params[b'listenerPort'] = listener_port

        body = {}
        if scheduler is not None:
            body['scheduler'] = compat.convert_to_string(scheduler)

        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              listener_port=int)
    def update_app_udp_listener(self, blb_id, listener_port,
                                scheduler=None, client_token=None,
                                config=None):
        """
        update a app udp listener rule with the specified options.

        :param blb_id:
                the id of blb which the listener work on
        :type blb_id:string

        :param listener_port:
                port to be linstened owned by listener
        :value 1-65535
        :type listener_port:int

        :param scheduler
              balancing algorithm
        :value 'RoundRobin'or'LeastConnection'or'Hash'
        :type scheduler:string

        :param client_token:
                If the clientToken is not specified by the user,
                a random String generated by default algorithm will be used.
        :type client_token: string

         :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'UDPlistener')
        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token
        params[b'listenerPort'] = listener_port

        body = {
            'scheduler': compat.convert_to_string(scheduler)
        }

        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body),
                                  params=params, config=config)

    @required(blb_id=(bytes, str),
              listener_port=int)
    def update_app_http_listener(self, blb_id, listener_port,
                                 scheduler=None, keep_session=None,
                                 keep_session_type=None,
                                 keep_session_timeout=None,
                                 keep_session_cookie_name=None,
                                 x_forward_for=None,
                                 server_timeout=None,
                                 redirect_port=None,
                                 client_token=None,
                                 config=None):
        """
        update a app http listener rule with the specified options.
        :param blb_id:
            The id of blb which the listener work on
        :type blb_id: string
        :param listener_port:
            Port to be linstened owned by listener
        :value 1-65535
        :type listener_port: int
        :param scheduler:
            Balancing algorithm
        :value 'RoundRobin' or 'LeastConnection' or 'Hash'
        :type scheduler: string
        :param keep_session:
            Whether to enable the session hold function, that is,
            the request sent by the same client will reach the
            same backend server
        :value true or false, default:false
        :type keep_session: bool
        :param keep_session_type:
            The cookie handling method maintained by the session,
            valid only if the session is held open
        :value 'insert' or 'rewrite', default:insert
        :type keep_session_type: string
        :param keep_session_timeout:
            The time the cookie is kept in session (in seconds),
            valid only if the session is held open
        :value 1-15552000, default:3600
        :type keep_session_timeout: int
        :param keep_session_cookie_name:
            The session keeps the name of the cookie that needs
            to be overridden,if and only if session persistence is
            enabled and keep_session_type="rewrite"
        :type keep_session_cookie_name: int
        :param x_forward_for:
            Whether to enable the real IP address of the client,
            the backend server can obtain the real address of the
            client through the X-Forwarded-For HTTP header.
        :value true or false, default: flase
        :type x_forward_for: bool
        :param server_timeout:
            Backend server maximum timeout (unit: second)
        :value 1-3600, default: 30
        :type server_timeout: int
        :param redirect_port:
            Forward the request received by this listener to the HTTPS
            listener, which is specified by the HTTPS listener.
        :type redirect_port: int
        :param client_token:
                If the clientToken is not specified by the user,
                a random String generated by default algorithm will be used.
        :type client_token: string
        :param config:
        :type config: baidubce.BceClientConfiguration
        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'HTTPlistener')
        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token
        params[b'listenerPort'] = listener_port

        body = {}
        if scheduler is not None:
            body['scheduler'] = compat.convert_to_string(scheduler)
        if keep_session is not None:
            body['keepSession'] = keep_session
        if keep_session_type is not None:
            body['keepSessionType'] = \
                compat.convert_to_string(keep_session_type)
        if keep_session_timeout is not None:
            body['keepSessionTimeout'] = keep_session_timeout
        if keep_session_cookie_name is not None:
            body['keepSessionCookieName'] = keep_session_cookie_name
        if x_forward_for is not None:
            body['xForwardFor'] = x_forward_for
        if server_timeout is not None:
            body['serverTimeout'] = server_timeout
        if redirect_port is not None:
            body['redirectPort'] = redirect_port
        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body),
                                  params=params, config=config)

    @required(blb_id=(bytes, str), listener_port=int)
    def update_app_https_listener(self, blb_id, listener_port,
                                  scheduler=None,
                                  keep_session=None,
                                  keep_session_type=None,
                                  keep_session_timeout=None,
                                  keep_session_cookie_name=None,
                                  x_forward_for=None,
                                  server_timeout=None,
                                  cert_ids=None,
                                  ie6_compatible=None,
                                  encryption_type=None,
                                  encryption_protocols=None,
                                  dual_auth=None,
                                  client_certIds=None,
                                  client_token=None,
                                  config=None):
        """
        update a app https listener rule with the specified options.
        :param blb_id:
            The id of blb which the listener work on
        :type blb_id: string
        :param listener_port:
            Port to be linstened owned by listener
        :value 1-65535
        :type listener_port: int
        :param scheduler:
            Balancing algorithm
        :value 'RoundRobin' or 'LeastConnection' or 'Hash'
        :type scheduler: string
        :param keep_session:
            Whether to enable the session hold function, that is, the request
            sent by the same client will reach the same backend server
        :value true or false, default: false
        :type keep_session: bool
        :param keep_session_type:
            The cookie handling method maintained by the session,
            valid only if the session is held open
        :value 'insert' or 'rewrite', default: insert
        :type keep_session_type: string
        :param keep_session_timeout:
            The time the cookie is kept in session (in seconds),
            valid only if the session is held open
        :value 1-15552000, default:3600
        :type keep_session_timeout: int
        :param keep_session_cookie_name:
            The session keeps the name of the cookie that needs to be
            overridden,if and only if session persistence is enabled and
            keep_session_type="rewrite"
        :type keep_session_cookie_name: int
        :param x_forward_for:
            Whether to enable the real IP address of the client,
            the backend server can obtain the real address of the client
            through the X-Forwarded-For HTTP header.
        :value true or false, default: False
        :type x_forward_for: bool
        :param server_timeout:
            Backend server maximum timeout (unit: second)
        :value 1-3600, default: 30
        :type server_timeout: int
        :param cert_ids:
            The SSL certificate to be loaded by the listener.
            Currently HTTPS listeners can only bind one SSL certificate.
        :type cert_ids:List<String>
        :param ie6_compatible:
            Is it compatible with IE6 HTTPS request
            (the protocol format is earlier SSL3.0, the security is poor)
        :value true or false, default: true
        :type ie6_compatible: bool
        :param encryption_type:
            Encryption options, support three types:
            compatibleIE/incompatibleIE/userDefind,
            corresponding to:
            IE-compatible encryption/disabled unsecure encryption/custom encryption,
            when encryptionType is valid and legitimate,
            ie6Compatible field transfer value will not take effect
        type: encryption_type:string
        :param encryption_protocols:
            When the encryptionType value is userDefind,
            the list of protocol types is a string list composed of four protocols:
            "sslv3", "tlsv10", "tlsv11", "tlsv12".
        type: encryption_protocols:list
        :param dual_auth:
            Whether to Open Two-way Authentication,
            default:false
        :type dual_auth: boolean
        :param client_certIds:
            When dualAuth is true, the loaded client certificate chain
        :type client_certIds: list
        :param client_token:
            If the clientToken is not specified by the user,
            a random String generated by default algorithm will be used.
        :type client_token: string
        :param config:
        :type config: baidubce.BceClientConfiguration
        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'HTTPSlistener')
        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token
        params[b'listenerPort'] = listener_port
        body = {}
        if scheduler is not None:
            body['scheduler'] = compat.convert_to_string(scheduler)
        if keep_session is not None:
            body['keepSession'] = keep_session
        if keep_session_type is not None:
            body['keepSessionType'] = \
                compat.convert_to_string(keep_session_type)
        if keep_session_timeout is not None:
            body['keepSessionTimeout'] = keep_session_timeout
        if keep_session_cookie_name is not None:
            body['keepSessionCookieName'] = keep_session_cookie_name
        if x_forward_for is not None:
            body['xForwardFor'] = x_forward_for
        if server_timeout is not None:
            body['serverTimeout'] = server_timeout
        if cert_ids is not None:
            body['certIds'] = cert_ids
        if ie6_compatible is not None:
            body['compatibleIE'] = ie6_compatible
        if encryption_type is not None:
            body['encryptionType'] = \
                compat.convert_to_string(encryption_type)
        if encryption_protocols is not None:
            body['encryptionProtocols'] = encryption_protocols
        if dual_auth is not None:
            body['dualAuth'] = dual_auth
        if client_certIds is not None:
            body['clientCertIds'] = client_certIds
        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str), listener_port=int)
    def update_app_ssl_listener(self, blb_id, listener_port,
                                scheduler=None,
                                cert_ids=None,
                                ie6_compatible=None,
                                encryption_type=None,
                                encryption_protocols=None,
                                dual_auth=None,
                                client_certIds=None,
                                client_token=None,
                                config=None):
        """
        update a app ssl listener rule with the specified options.

        :param blb_id:
            The id of blb which the listener work on
        :type blb_id: string

        :param listener_port:
            port to be linstened owned by listener
        :value 1-65535
        :type listener_port: int

        :param scheduler:
            balancing algorithm
        :value 'RoundRobin' or 'LeastConnection'
        :type scheduler: string

        :param cert_ids:
            The SSL certificate to be loaded by the listener.
            Currently HTTPS listeners can only bind one SSL certificate.
        :type cert_ids: List<String>

        :param ie6_compatible:
            compatible with IE6 HTTPS request
            (the protocol format is earlier SSL3.0, the security is poor)
        :value true or false, default: true
        :type ie6_compatible: bool

        :param encryption_type:
            Encryption options, support three types:
            compatibleIE/incompatibleIE/userDefind,
            corresponding to:
            IE-compatible encryption/disabled unsecure encryption/custom encryption,
            when encryptionType is valid and legitimate,
            ie6Compatible field transfer value will not take effect
        type: encryption_type:string

        :param encryption_protocols:
            When the encryptionType value is userDefind,
            the list of protocol types is a string list composed of four protocols:
            "sslv3", "tlsv10", "tlsv11", "tlsv12".
        type: encryption_protocols:list

        :param dual_auth:
            Whether to Open Two-way Authentication,
            default:false
        :type dual_auth: boolean

        :param client_certIds:
            When dualAuth is true, the loaded client certificate chain
        :type client_certIds: list

        :param client_token:
            If the clientToken is not specified by the user,
            a random String generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'SSLlistener')
        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token
        params[b'listenerPort'] = listener_port
        body = {}
        if scheduler is not None:
            body['scheduler'] = compat.convert_to_string(scheduler)
        if cert_ids is not None:
            body['certIds'] = cert_ids
        if ie6_compatible is not None:
            body['compatibleIE'] = ie6_compatible
        if encryption_type is not None:
            body['encryptionType'] = \
                compat.convert_to_string(encryption_type)
        if encryption_protocols is not None:
            body['encryptionProtocols'] = encryption_protocols
        if dual_auth is not None:
            body['dualAuth'] = dual_auth
        if client_certIds is not None:
            body['clientCertIds'] = client_certIds

        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body),
                                  params=params, config=config)

    @required(blb_id=(bytes, str))
    def describe_app_tcp_listener(self, blb_id, listener_port=None,
                                  marker=None, max_keys=None,
                                  config=None):
        """
        get app tcp listeners identified by bibID

        :param blb_id
             the id of blb which the listener work on
        :type blb_id:string

        :param listener_port
             The listener port to query
        :type listener_port:int

        :param marker
            The optional parameter marker specified in the
            original request to specify
            where in the results to begin listing.
            Together with the marker, specifies the list result
            which listing should begin.
            If the marker is not specified, the list result will
            listing from the first one.
        :type marker: string

        :param max_keys
            The optional parameter to specifies the max number of
            list result to return.
            The default value is 1000.
        :type max_keys: int

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'TCPlistener')
        params = {}

        if listener_port is not None:
            params[b'listenerPort'] = listener_port
        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys

        return self._send_request(http_methods.GET, path,
                                  params=params, config=config)

    @required(blb_id=(bytes, str))
    def describe_app_udp_listener(self, blb_id, listener_port=None,
                                  marker=None, max_keys=None,
                                  config=None):
        """
        get app udp listeners identified by bibID

        :param blb_id
             the id of blb which the listener work on
        :type blb_id:string

        :param listener_port
             The listener port to query
        :type listener_port:int

        :param marker
            The optional parameter marker specified in the original
            request to specify where in the results to begin listing.
            Together with the marker, specifies the list result which
            listing should begin.
            If the marker is not specified, the list result will
            listing from the first one.
        :type marker: string

        :param max_keys
        The optional parameter to specifies the max number of
        list result to return.
            The default value is 1000.
        :type max_keys: int

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'UDPlistener')
        params = {}

        if listener_port is not None:
            params[b'listenerPort'] = listener_port
        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys

        return self._send_request(http_methods.GET, path,
                                  params=params, config=config)

    @required(blb_id=(bytes, str))
    def describe_app_http_listener(self, blb_id, listener_port=None,
                                   marker=None, max_keys=None,
                                   config=None):
        """
        get app http listeners identified by bibID

        :param blb_id
             the id of blb which the listener work on
        :type blb_id:string

        :param listener_port
             The listener port to query
        :type listener_port:int

        :param marker
            The optional parameter marker specified in the original
            request to specify where in the results to begin listing.
            Together with the marker, specifies the list result which
            listing should begin.
            If the marker is not specified, the list result will listing
            from the first one.
        :type marker: string

        :param max_keys
            The optional parameter to specifies the max number of list
            result to return.
            The default value is 1000.
        :type max_keys: int

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'HTTPlistener')
        params = {}

        if listener_port is not None:
            params[b'listenerPort'] = listener_port
        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys

        return self._send_request(http_methods.GET, path,
                                  params=params, config=config)

    @required(blb_id=(bytes, str))
    def describe_app_https_listener(self, blb_id, listener_port=None,
                                    marker=None, max_keys=None,
                                    config=None):
        """
        get app https listeners identified by bibID

        :param blb_id
             the id of blb which the listener work on
        :type blb_id:string

        :param listener_port
             The listener port to query
        :type listener_port:int

        :param marker
            The optional parameter marker specified in the original
            request to specify where in the results to begin listing.
            Together with the marker, specifies the list result which
            listing should begin.
            If the marker is not specified, the list result will listing
            from the first one.
        :type marker: string

        :param max_keys
            The optional parameter to specifies the max number of list
            result to return.
            The default value is 1000.
        :type max_keys: int

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'HTTPSlistener')
        params = {}

        if listener_port is not None:
            params[b'listenerPort'] = listener_port
        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys

        return self._send_request(http_methods.GET, path,
                                  params=params, config=config)

    @required(blb_id=(bytes, str))
    def describe_app_ssl_listener(self, blb_id, listener_port=None,
                              marker=None, max_keys=None, config=None):
        """
        get app ssl listeners identified by bibID

        :param blb_id
             the id of blb which the listener work on
        :type blb_id:string

        :param listener_port
             The listener port to query
        :type listener_port:int

        :param marker
            The optional parameter marker specified in the original
            request to specify where in the results to begin listing.
            Together with the marker, specifies the list result which
            listing should begin.
            If the marker is not specified, the list result will listing
            from the first one.
        :type marker: string

        :param max_keys
            The optional parameter to specifies the max number of list
            result to return.
            The default value is 1000.
        :type max_keys: int

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'SSLlistener')
        params = {}

        if listener_port is not None:
            params[b'listenerPort'] = listener_port
        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys

        return self._send_request(http_methods.GET, path,
                                  params=params, config=config)

    @required(blb_id=(bytes, str),
              portList=list)
    def delete_app_listeners(self, blb_id, portList,
                             client_token=None,
                             config=None):
        """
        Release app listener under the specified LoadBalancer,
        the listener is specified by listening to the port.

        :param blb_id:
            id of LoadBalancer
        :type blb_id:string

        :param portList:
            The ports of listeners to be released
        :type portList:list<int>

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'listener')
        params = {}

        params[b'batchdelete'] = None

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {}
        body['portList'] = portList

        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              listener_port=int,
              app_policy_vos=list)
    def create_policys(self, blb_id, listener_port,
                       app_policy_vos, client_token=None,
                       config=None):
        """
        Create policys.

        :param blb_id:
            the id of blb which the listener work on
        :type blb_id: string

        :param listener_port:
            port to be linstened owned by listener
        :value 1-65535
        :type listener_port: int

        :param app_policy_vos
            policy list the listener binds.
            If the listener type is TCP,
            there is only one policy
            and only the full match is supported.
        https://cloud.baidu.com/doc/BLB/API.html#AppPolicy
        :type app_policy_vos: list<AppPolicy>

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'policys')
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'listenerPort': listener_port,
            'appPolicyVos': app_policy_vos
        }

        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              listener_port=int)
    def describe_policys(self, blb_id, listener_port,
                         marker=None, max_keys=None,
                         config=None):
        """
        get policys

        :param blb_id
             the id of blb which the listener work on
        :type blb_id:string

        :param listener_port
             The listener port used by listener
        :type listener_port:int

        :param marker
            The optional parameter marker specified in the original
            request to specify where in the results to begin listing.
            Together with the marker, specifies the list result which
            listing should begin.
            If the marker is not specified, the list result will listing
            from the first one.
        :type marker: string

        :param max_keys
            The optional parameter to specifies the max number of list
            result to return.
            The default value is 1000.
        :type max_keys: int

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'policys')
        params = {}
        params[b'port'] = listener_port
        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys

        return self._send_request(http_methods.GET, path,
                                  params=params, config=config)

    @required(blb_id=(bytes, str),
              listener_port=int,
              policys_list=list)
    def delete_policys(self, blb_id, listener_port,
                         policys_list,
                         client_token=None, config=None):
        """
        Release the listener under the specified LoadBalancer,
        the listener is specified by listening to the port.

        :param blb_id:
            id of LoadBalancer
        :type blb_id:string

        :param listener_port
             The listener port used by listener
        :type listener_port:int

         :param policys_list
             All policy identifiers to be released
        :type policys_list:list<str>

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'policys')
        params = {}

        params[b'batchdelete'] = None

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'port': listener_port,
            'policyIdList': policys_list
        }

        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    """
        ServerGroup API
    """

    @required(blb_id=(bytes, str))
    def create_app_server_group(self, blb_id,
                                name=None,
                                desc=None,
                                backend_server_list=None,
                                client_token=None,
                                config=None):
        """
        create server group for the specified LoadBalancer,
        support batch add

        :param blb_id:
            id of LoadBalancer
        :type blb_id:string

        :param name:
            name of server group
        :type name:string

        :param desc:
            description of server group
        :type desc:string

        :param backend_server_list
            List of backend servers to be added
        https://cloud.baidu.com/doc/BLB/API.html#AppBackendServer
        :type backend_server_list:List<AppBackendServer>

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'appservergroup')
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {}
        if name is not None:
            body['name'] = compat.convert_to_string(name)
        if desc is not None:
            body['desc'] = compat.convert_to_string(desc)
        if backend_server_list is not None:
            body['backendServerList'] = backend_server_list

        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              sg_id=(bytes, str))
    def update_app_server_group(self, blb_id, sg_id,
                                name=None,
                                desc=None,
                                client_token=None,
                                config=None):
        """
        update the information of the app server group
        of the specified LoadBalancer

        :param blb_id:
            id of LoadBalancer
        :type blb_id:string

        :param sg_id:
            id of the server group to be updated
        :type sg_id:string

        :param name:
            name of server group
        :type name:string

        :param desc:
            description of server group
        :type desc:string

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'appservergroup')
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {}
        body['sgId'] = compat.convert_to_string(sg_id)
        if name is not None:
            body['name'] = compat.convert_to_string(name)
        if desc is not None:
            body['desc'] = compat.convert_to_string(desc)

        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str))
    def describe_app_server_group(self, blb_id,
                                  name=None,
                                  exactly_match=None,
                                  marker=None,
                                  max_keys=None, config=None):
        """
        Query the imformation of app server group
        of the specified LoadBalancer

        :param blb_id:
            Id of LoadBalancer
        :type blb_id:string

        :param name:
            name of server group
        :type name:string

        :param exactly_match:
            Set whether the name matches globally
        :type exactly_match:boolean

        :param marker:
            The optional parameter marker specified in the original
            request to specify where in the results to begin listing.
            Together with the marker, specifies the list result which
            listing should begin. If the marker is not specified,
            the list result will listing from the first one.
        :type marker: string

        :param max_keys:
            The optional parameter to specifies the max number of
            list result to return.
            The default value is 1000.
        :type max_keys: int

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'appservergroup')
        params = {}


        if name is not None:
            params[b'name'] = name
        if exactly_match is not None:
            params[b'exactlyMatch'] = exactly_match
        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys

        return self._send_request(http_methods.GET, path, params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              sg_id=(bytes, str))
    def delete_app_server_group(self, blb_id, sg_id,
                                client_token=None,
                                config=None):
        """
        delete the app server group of the specified LoadBalancer,

        :param blb_id:
                id of LoadBalancer
        :type blb_id:string

        :param sg_id:
            id of the server group to be updated
        :type sg_id:string

        :param client_token:
                If the clientToken is not specified by the user,
                a random String generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'appservergroup')
        params = {}

        params[b'delete'] = None
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {}
        body['sgId'] = compat.convert_to_string(sg_id)

        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              sg_id=(bytes, str), port=int,
              protocol_type=(bytes, str))
    def create_app_server_group_port(self, blb_id, sg_id,
                                     port, protocol_type,
                                     health_check=None,
                                     health_check_port=None,
                                     health_check_urlpath=None,
                                     health_check_timeout_insecond=None,
                                     health_check_interval_insecond=None,
                                     health_check_down_retry=None,
                                     health_check_up_retry=None,
                                     health_check_normal_status=None,
                                     client_token=None,
                                     config=None):
        """
        create server group for the specified LoadBalancer,
        support batch add
        :param blb_id:
            id of LoadBalancer
        :type blb_id:string
        :param sg_id:
            id of the server group
        :type sg_id:string
        :param port:
            Port number, integer between 1 and 65535
        :type port:string
        :param protocol_type:
            Protocol type of listening port, "TCP"/"UDP"/"HTTP"
        :type protocol_type:string
        :param health_check:
            Health check protocol
        :value 'HTTP' or 'TCP',default:'HTTP'
        :type health_check: string
        :param health_check_port:
            Health check port, the default is the same as port
        :type health_check_port: int
        :param health_check_urlpath:
            Health check URI, default '/'.
            Effective when the health check protocol is "HTTP"
        :type health_check_urlpath: string
         :param health_check_timeout_insecond:
            Health check timeout (unit: second)
        :value 1-60, default: 3
        :type health_check_timeout_insecond: int
        :param health_check_interval_insecond:
            Health check interval (unit: second)
        :value 1-10, default: 3
        :type health_check_interval_insecond: int
        :param health_check_down_retry:
            The unhealthy down retry, that is, how many consecutive health
            check failures, shields the backend server.
        :value 2-5, default: 3
        :type health_check_down_retry: int
        :param health_check_up_retry:
            Health up retry, that is, how many consecutive health checks
            are successful, then re-use the back-end server
        :value:2-5, default: 3
        :type health_check_up_retry: int
        :param health_check_normal_status:
            The HTTP status code when the health check is normal supports
            a combination of five types of status codes,
            such as "http_1xx|http_2xx", Effective when the health check
            protocol is "HTTP"
        :value default: http_2xx|http_3xx
        :type health_check_normal_status: string
        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string
        :param config:
        :type config: baidubce.BceClientConfiguration
        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'appservergroupport')
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'sgId': compat.convert_to_string(sg_id),
            'port': port,
            'type': compat.convert_to_string(protocol_type)
        }
        if health_check is not None:
            body['healthCheck'] = compat.convert_to_string(health_check)
        if health_check_port is not None:
            body['healthCheckPort'] = health_check_port
        if health_check_urlpath is not None:
            body['healthCheckUrlPath'] = \
                compat.convert_to_string(health_check_urlpath)
        if health_check_timeout_insecond is not None:
            body['healthCheckTimeoutInSecond'] = health_check_timeout_insecond
        if health_check_interval_insecond is not None:
            body['healthCheckIntervalInSecond'] = health_check_interval_insecond
        if health_check_down_retry is not None:
            body['healthCheckDownRetry'] = health_check_down_retry
        if health_check_up_retry is not None:
            body['healthCheckUpRetry'] = health_check_up_retry
        if health_check_normal_status is not None:
            body['healthCheckNormalStatus'] = \
                compat.convert_to_string(health_check_normal_status)
        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              sg_id=(bytes, str),
              port_id=(bytes, str))
    def update_app_server_group_port(self, blb_id, sg_id, port_id,
                                     health_check=None,
                                     health_check_port=None,
                                     health_check_urlpath=None,
                                     health_check_timeout_insecond=None,
                                     health_check_interval_insecond=None,
                                     health_check_down_retry=None,
                                     health_check_up_retry=None,
                                     health_check_normal_status=None,
                                     client_token=None,
                                     config=None):
        """
        update server group for the specified LoadBalancer,
        support batch add

        :param blb_id:
            id of LoadBalancer
        :type blb_id:string

        :param sg_id:
            id of the server group
        :type sg_id:string

        :param port_id:
            The id of the server group port to be updated
        :type port_id:string

        :param health_check:
            Health check protocol
        :value 'HTTP' or 'TCP',default:'HTTP'
        :type health_check: string

        :param health_check_port:
            Health check port, the default is the same as port
        :type health_check_port: int

        :param health_check_urlpath:
            Health check URI, default '/'.
            Effective when the health check protocol is "HTTP"
        :type health_check_urlpath: string

         :param health_check_timeout_insecond:
            Health check timeout (unit: second)
        :value 1-60, default: 3
        :type health_check_timeout_insecond: int

        :param health_check_interval_insecond:
            Health check interval (unit: second)
        :value 1-10, default: 3
        :type health_check_interval_insecond: int

        :param health_check_down_retry:
            The unhealthy down retry, that is, how many consecutive health
            check failures, shields the backend server.
        :value 2-5, default: 3
        :type health_check_down_retry: int

        :param health_check_up_retry:
            Health up retry, that is, how many consecutive health checks
            are successful, then re-use the back-end server
        :value:2-5, default: 3
        :type health_check_up_retry: int

        :param health_check_normal_status:
            The HTTP status code when the health check is normal supports
            a combination of five types of status codes,
            such as "http_1xx|http_2xx", Effective when the health check
            protocol is "HTTP"
        :value default: http_2xx|http_3xx
        :type health_check_normal_status: string

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'appservergroupport')
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'sgId': compat.convert_to_string(sg_id),
            'portId': compat.convert_to_string(port_id)
        }
        if health_check is not None:
            body['healthCheck'] = compat.convert_to_string(health_check)
        if health_check_port is not None:
            body['healthCheckPort'] = health_check_port
        if health_check_urlpath is not None:
            body['healthCheckUrlPath'] = \
                compat.convert_to_string(health_check_urlpath)
        if health_check_timeout_insecond is not None:
            body['healthCheckTimeoutInSecond'] = health_check_timeout_insecond
        if health_check_interval_insecond is not None:
            body['healthCheckIntervalInSecond'] = health_check_interval_insecond
        if health_check_down_retry is not None:
            body['healthCheckDownRetry'] = health_check_down_retry
        if health_check_up_retry is not None:
            body['healthCheckUpRetry'] = health_check_up_retry
        if health_check_normal_status is not None:
            body['healthCheckNormalStatus'] = \
                compat.convert_to_string(health_check_normal_status)

        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              sg_id=(bytes, str),
              port_list=list)
    def delete_app_server_group_port(self, blb_id, sg_id,
                                     port_list,
                                     client_token=None, config=None):
        """
        delete server group of the specified LoadBalancer,

        :param blb_id:
            id of LoadBalancer
        :type blb_id:string

        :param sg_id:
            id of the server group
        :type sg_id:string

        :param port_list:
            The ports of listeners to be released
        :type port_list:list<string>

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'appservergroupport')
        params = {}

        params[b'batchdelete'] = None

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'sgId': compat.convert_to_string(sg_id),
            'portIdList': port_list
        }

        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              sg_id=(bytes, str),
              backend_server_list=list)
    def create_app_blb_rs(self, blb_id, sg_id,
                          backend_server_list,
                          client_token=None,
                          config=None):
        """
        Add backend server for the specified LoadBalancer and server group,
        support batch add

        :param blb_id:
                id of LoadBalancer
        :type blb_id:string

        :param sg_id:
            id of the server group
        :type sg_id:string

        :param backend_server_list
                List of backend servers to be added
        https://cloud.baidu.com/doc/BLB/API.html#AppBackendServer
        :type backend_server_list:List<AppBackendServer>

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'blbrs')
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'sgId': compat.convert_to_string(sg_id),
            'backendServerList': backend_server_list
        }

        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              sg_id=(bytes, str),
              backend_server_list=list)
    def update_app_blb_rs(self, blb_id, sg_id,
                          backend_server_list,
                          client_token=None,
                          config=None):
        """
        update backend server for the specified LoadBalancer and server group,
        support batch update

        :param blb_id:
                id of LoadBalancer
        :type blb_id:string

        :param sg_id:
            id of the server group
        :type sg_id:string

        :param backend_server_list
                List of backend servers to be added
        https://cloud.baidu.com/doc/BLB/API.html#AppBackendServer
        :type backend_server_list:List<AppBackendServer>

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'blbrs')
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'sgId': compat.convert_to_string(sg_id),
            'backendServerList': backend_server_list
        }

        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              sg_id=(bytes, str))
    def describe_app_blb_rs(self, blb_id, sg_id,
                            marker=None, max_keys=None,
                            config=None):
        """
        Query the list of backend servers under the specified LoadBalancer
        and server group

        :param blb_id:
            Id of LoadBalancer
        :type blb_id:string

        :param sg_id:
            id of the server group
        :type sg_id:string

        :param marker:
            The optional parameter marker specified in the original
            request to specify where in the results to begin listing.
            Together with the marker, specifies the list result which
            listing should begin. If the marker is not specified,
            the list result will listing from the first one.
        :type marker: string

        :param max_keys:
            The optional parameter to specifies the max number of
            list result to return.
            The default value is 1000.
        :type max_keys: int

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'blbrs')
        params = {}
        params[b'sgId'] = compat.convert_to_string(sg_id)

        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys

        return self._send_request(http_methods.GET, path, params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              sg_id=(bytes, str),
              backend_server_list=list)
    def delete_app_blb_rs(self, blb_id, sg_id,
                          backend_server_list,
                          client_token=None,
                          config=None):
        """
        delete backend server for the specified LoadBalancer and server group,
        support batch delete

        :param blb_id:
                id of LoadBalancer
        :type blb_id:string

        :param sg_id:
            id of the server group
        :type sg_id:string

        :param backend_server_list
                List of backend servers to be deleted
        :type backend_server_list:List<string>

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'blbrs')
        params = {}

        params[b'batchdelete'] = None
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'sgId': compat.convert_to_string(sg_id),
            'backendServerIdList': backend_server_list
        }

        return self._send_request(http_methods.PUT, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              sg_id=(bytes, str))
    def describe_rs_mount(self, blb_id, sg_id, config=None):
        """
        describe servers of specific server group

        :param blb_id:
                id of LoadBalancer
        :type blb_id:string

        :param sg_id:
            id of the server group
        :type sg_id:string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'blbrsmount')

        params = {
            'sgId': compat.convert_to_string(sg_id)
        }

        return self._send_request(http_methods.GET, path, params=params,
                                  config=config)

    @required(blb_id=(bytes, str),
              sg_id=(bytes, str))
    def describe_rs_unmount(self, blb_id, sg_id, config=None):
        """
        describe servers of specific server group

        :param blb_id:
                id of LoadBalancer
        :type blb_id:string

        :param sg_id:
            id of the server group
        :type sg_id:string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = utils.append_uri(self.version, 'appblb', blb_id, 'blbrsunmount')

        params = {
            'sgId': compat.convert_to_string(sg_id)
        }

        return self._send_request(http_methods.GET, path, params=params,
                                  config=config)


def generate_client_token_by_uuid():
    """
    The default method to generate the random string for client_token
    if the optional parameter client_token is not specified by the user.

    :return:
    :rtype string
    """
    return str(uuid.uuid4())


generate_client_token = generate_client_token_by_uuid



# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client class for Media.
"""

import copy
import json
import logging
#from baidubce import bce_base_client
from baidubce.bce_base_client import BceBaseClient
from baidubce.auth import bce_v1_signer
from baidubce.bce_response import BceResponse
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_content_types
from baidubce.http import http_headers
from baidubce.http import http_methods
from baidubce.exception import BceClientError
from baidubce.exception import BceServerError
from baidubce.services.media import media_handler
from baidubce.utils import required

_logger = logging.getLogger(__name__)


class MediaClient(BceBaseClient):
    """
    sdk client
    """

    prefix = '/v3'

    def __init__(self, config=None):
        BceBaseClient.__init__(self, config)

    @required(pipeline_name=(bytes, str))
    def list_jobs(self,
                  pipeline_name,
                  job_status=None,
                  begin=None,
                  end=None,
                  config=None):
        """
        List jobs by pipelineName and jobStatus

        :param pipelineName: The pipeline name
        :type  pipelineName: string or unicode
        :param job_status: The job status, will not filter if None
        :type  job_status: string or unicode
        :param begin: The createTime should be later than or equals with begin, will not check if None
        :type  begin: string or unicode
        :param end: The createTime should be earlier than or equals with end, will not check if None
        :type  end: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if pipeline_name == '':
            raise BceClientError('pipeline_name can\'t be empty string')
        my_params={
                'pipelineName': pipeline_name
        }
        if job_status is not None:
            my_params["jobStatus"] = job_status
        if begin is not None:
            my_params["begin"] = begin
        if end is not None:
            my_params["end"] = end
        return self._send_request(
            http_methods.GET, 
            '/job/transcoding',
            params=my_params,
            config=config)

    @required(pipeline_name=(bytes, str), source=dict, target=dict)
    def create_job(self, pipeline_name, source, target, config=None):
        """
        Create a job

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param source: source
        :type source: array
        :param target: source
        :type target: array
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if pipeline_name == '':
            raise BceClientError('pipeline_name can\'t be empty string')
        return self._send_request(
            http_methods.POST,
            '/job/transcoding',
            body=json.dumps({
                'pipelineName': pipeline_name,
                'source': source,
                'target': target}),
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            config=config)

    @required(job_id=(bytes, str))
    def get_job(self, job_id, config=None):
        """
        Get the specific job information

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if job_id == '':
            raise BceClientError('job_id can\'t be empty string')
        return self._send_request(
            http_methods.GET, 
            '/job/transcoding/' + job_id,
            config=config)

    @required(job_id=(bytes, str))
    def get_transcoding_encryption_key(self, job_id, config=None):
        """
        Get transcoding job encryption key

        :param job_id: The job Id want to query transcoding encryption key.
        :type job_id: string
        :param config: None
        :type config: BceClientConfiguration
        :returns: Response Object contains transcoding encryption Aes key.
        :rtype: baidubce.bce_response.BceResponse
        """
        if job_id == '':
            raise BceClientError('job_id can\'t be empty string')
        return self._send_request(
            http_methods.GET, 
            '/transcoding/key/' + job_id,
            body_parser=media_handler.parse_secret_key_response,
            config=config)

    @required(bucket=(bytes, str), key=(bytes, str))
    def get_mediainfo_of_file(self, bucket, key, config=None):
        """
        Get the media info of media information

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if bucket == '':
            raise BceClientError('bucket can\'t be empty string')
        if key == '':
            raise BceClientError('key can\'t be empty string')
        return self._send_request(
            http_methods.GET,
            '/mediainfo',
            params={
                'bucket': bucket,
                'key': key},
            config=config)

    def list_pipelines(self, config=None):
        """
        List pipelines

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        return self._send_request(http_methods.GET, '/pipeline', config=config)

    @required(
        pipeline_name=(bytes, str),
        source_bucket=(bytes, str),
        target_bucket=(bytes, str))
    def create_pipeline(
            self,
            pipeline_name,
            source_bucket,
            target_bucket,
            description=None,
            pipeline_config=None,
            config=None):
        """
        Create a pipeline

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        body_content = {}
        if description is not None:
            body_content['description'] = description
        else:
            body_content['description'] = ""
        if pipeline_config is not None:
            body_content['config'] = pipeline_config
        if pipeline_name == '':
            raise BceClientError('pipeline_name can\'t be empty string')
        if source_bucket == '':
            raise BceClientError('source_bucket can\'t be empty string')
        if target_bucket == '':
            raise BceClientError('target_bucket can\'t be empty string')
        body_content['pipelineName'] = pipeline_name
        body_content['sourceBucket'] = source_bucket
        body_content['targetBucket'] = target_bucket
        return self._send_request(
            http_methods.POST,
            '/pipeline',
            body=json.dumps(body_content),
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            config=config)

    @required(pipeline_name=(bytes, str), pipeline=(BceResponse, dict))
    def update_pipeline(
            self,
            pipeline_name,
            pipeline,
            config=None):
        """
        Update a pipeline

        :param pipeline_name: the preset name
        :type pipeline_name: string
        :param pipeline: the update body
        :type pipeline: Object
        :returns:
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        return self._send_request(
            http_methods.PUT,
            '/pipeline/' + pipeline_name,
            body=json.dumps(pipeline, default=lambda o: o.__dict__),
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            config=config)

    @required(pipelien_name=(bytes, str))
    def get_pipeline_for_update(self, pipeline_name, config=None):
        """
        Get the specific pipeline information without converting camel key to a "pythonic" name

        :param pipeline_name: The pipeline name
        :type pipeline_name: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if pipeline_name == '':
            raise BceClientError('pipeline_name can\'t be empty string')
        return self._send_request(http_methods.GET, '/pipeline/' + pipeline_name,
                                  config=config, body_parser=media_handler.parse_json)

    @required(pipeline_name=(bytes, str))
    def get_pipeline(self, pipeline_name, config=None):
        """
        Get the specific pipeline information

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if pipeline_name == '':
            raise BceClientError('pipeline_name can\'t be empty string')
        return self._send_request(http_methods.GET, '/pipeline/' + pipeline_name, config=config)

    @required(pipeline_name=(bytes, str))
    def delete_pipeline(self, pipeline_name, config=None):
        """
        Delete the specific pipeline

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if pipeline_name == '':
            raise BceClientError('pipeline_name can\'t be empty string')
        return self._send_request(http_methods.DELETE, '/pipeline/' + pipeline_name, config=config)

    def list_presets(self, config=None):
        """
        List presets

        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        return self._send_request(http_methods.GET, '/preset', config=config)

    @required(preset_name=(bytes, str), container=(bytes, str))
    def create_preset(
            self,
            preset_name,
            container,
            transmux=None,
            description=None,
            clip=None,
            audio=None,
            video=None,
            encryption=None,
            watermark_id=None,
            extra_config=None,
            trans_config=None,
            config=None):
        """
        Create a preset

        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """

        body_content = dict()
        if preset_name == '':
            raise BceClientError('preset_name can\'t be empty string')
        if container == '':
            raise BceClientError('container can\'t be empty string')
        body_content['presetName'] = preset_name
        body_content['container'] = container
        if description is None:
            body_content['description'] = ''
        else:
            body_content['description'] = description
        if transmux is not None:
            body_content['transmux'] = transmux
        if clip is not None:
            body_content['clip'] = clip
        if audio is not None:
            body_content['audio'] = audio
        if video is not None:
            body_content['video'] = video
        if encryption is not None:
            body_content['encryption'] = encryption
        if watermark_id is not None:
            body_content['watermarkId'] = watermark_id
        if extra_config is not None:
            body_content['extraCfg'] = extra_config
        if trans_config is not None:
            body_content['transCfg'] = trans_config
        return self._send_request(
            http_methods.POST,
            '/preset',
            body=json.dumps(body_content),
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            config=config)

    @required(preset_name=(bytes, str), preset=(BceResponse, dict))
    def update_preset(
            self,
            preset_name,
            preset,
            config=None):
        """
        Update a preset

        :param preset_name: the preset name
        :type preset_name: string
        :param preset: the update body
        :type preset: Object
        :returns:
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        return self._send_request(
            http_methods.PUT,
            '/preset/' + preset_name,
            body=json.dumps(preset, default=lambda o: o.__dict__),
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            config=config)

    @required(preset_name=(bytes, str))
    def get_preset(self, preset_name, config=None):
        """
        Get the specific preset information

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if preset_name == '':
            raise BceClientError('preset_name can\'t be empty string')
        return self._send_request(http_methods.GET, '/preset/' + preset_name, config=config)

    @required(preset_name=(bytes, str))
    def get_preset_for_update(self, preset_name, config=None):
        """
        Get the specific preset information without converting camel key to a "pythonic" name

        :param preset_name: The preset name
        :type preset_name: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if preset_name == '':
            raise BceClientError('preset_name can\'t be empty string')
        return self._send_request(http_methods.GET, '/preset/' + preset_name,
                                  config=config, body_parser=media_handler.parse_json)

    @required(preset_name=(bytes, str))
    def delete_preset(self, preset_name, config=None):
        """
        Delete a preset

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if preset_name == '':
            raise BceClientError('preset_name can\'t be empty string')
        return self._send_request(http_methods.DELETE, '/preset/' + preset_name, config=config)

    @required(pipeline_name=(bytes, str), source=dict)
    def create_thumbnail_job(self, pipeline_name, source, target=None, capture=None, config=None):
        """
        Create thumbnail job

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        body_content = {}
        if pipeline_name == '':
            raise BceClientError('pipeline_name can\'t be empty string')
        if target is not None:
            body_content['target'] = target
        if capture is not None:
            body_content['capture'] = capture
        body_content['pipelineName'] = pipeline_name
        body_content['source'] = source
        return self._send_request(
            http_methods.POST,
            '/job/thumbnail',
            body=json.dumps(body_content),
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            config=config)

    @required(job_id=(bytes, str))
    def get_thumbnail_job(self, job_id, config=None):
        """
        Get thumbnail job

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if job_id == '':
            raise BceClientError('job_id can\'t be empty string')
        return self._send_request(http_methods.GET, '/job/thumbnail/' + job_id, config=config)

    @required(pipeline_name=(bytes, str))
    def list_thumbnail_jobs_by_pipeline(self,
                                        pipeline_name,
                                        job_status=None,
                                        begin=None,
                                        end=None,
                                        config=None):
        """
        List thumbnail jobs by pipelineName

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param job_status: The thumbnail job status, will not filter if None
        :type  job_status: string or unicode
        :param begin: The createTime should be later than or equals with begin, will not check if None
        :type  begin: string or unicode
        :param end: The createTime should be earlier than or equals with end, will not check if None
        :type  end: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        
        if pipeline_name == '':
            raise BceClientError('pipeline_name can\'t be empty string')
        my_params={
                'pipelineName': pipeline_name
        }
        if job_status is not None:
            my_params["jobStatus"] = job_status
        if begin is not None:
            my_params["begin"] = begin
        if end is not None:
            my_params["end"] = end
        return self._send_request(
            http_methods.GET,
            '/job/thumbnail',
            params=my_params,
            config=config)

    @required(
        bucket=(bytes, str),
        key=(bytes, str),
        vertical_offset_in_pixel=int,
        hori_offset_in_pixel=int)
    def create_watermark(
            self,
            bucket,
            key,
            vertical_alignment=None,
            horizontal_alignment=None,
            vertical_offset_in_pixel=None,
            hori_offset_in_pixel=None,
            config=None):
        """
        Create watermark

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if bucket == '':
            raise BceClientError('bucket can\'t be empty string')
        if key == '':
            raise BceClientError('key can\'t be empty string')
        body_content = {}
        body_content['bucket'] = bucket
        body_content['key'] = key
        if vertical_alignment is not None:
            body_content['verticalAlignment'] = vertical_alignment
        if horizontal_alignment is not None:
            body_content['horizontalAlignment'] = horizontal_alignment
        if vertical_offset_in_pixel is not None:
            body_content['verticalOffsetInPixel'] = vertical_offset_in_pixel
        if hori_offset_in_pixel is not None:
            body_content['horizontalOffsetInPixel'] = hori_offset_in_pixel
        return self._send_request(
            http_methods.POST,
            '/watermark',
            body=json.dumps(body_content),
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            config=config)

    @required(watermark_id=(bytes, str))
    def get_watermark(self, watermark_id, config=None):
        """
        Get the specific watermark information

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if watermark_id == '':
            raise BceClientError('watermark_id can\'t be empty string')
        return self._send_request(http_methods.GET, '/watermark/' + watermark_id, config=config)

    def list_watermarks(self, config=None):
        """
        List watermarks

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        return self._send_request(http_methods.GET, '/watermark', config=config)

    @required(watermark_id=(bytes, str))
    def delete_watermark(self, watermark_id, config=None):
        """
        Delete the specific watermark

        :param pipelineName: The pipeline name
        :type pipelineName: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if watermark_id == '':
            raise BceClientError('watermark_id can\'t be empty string')
        return self._send_request(http_methods.DELETE, '/watermark/' + watermark_id, config=config)

    def list_notifications(self, config=None):
        """
        List notifications

        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        return self._send_request(http_methods.GET, '/notification', config=config)

    @required(
        name=(bytes, str),
        endpoint=(bytes, str))
    def create_notification(
            self,
            name,
            endpoint,
            config=None):
        """
        Create a notification

        :param name: The notification name
        :type name: string or unicode
        :param endpoint: The endpoint
        :type endpoint: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if name == '':
            raise BceClientError('name can\'t be empty string')
        if endpoint == '':
            raise BceClientError('endpoint can\'t be empty string')
        body_content = {'name': name,
                        'endpoint': endpoint}
        return self._send_request(
            http_methods.POST,
            '/notification',
            body=json.dumps(body_content),
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            config=config)

    @required(name=(bytes, str))
    def get_notification(self, name, config=None):
        """
        Get the specific notification information

        :param name: The notification name
        :type name: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if name == '':
            raise BceClientError('name can\'t be empty string')
        return self._send_request(http_methods.GET, '/notification/' + name, config=config)

    @required(name=(bytes, str))
    def delete_notification(self, name, config=None):
        """
        Delete the specific notification

        :param name: The notification name
        :type name: string or unicode
        :param config: None
        :type config: BceClientConfiguration
        :returns:
        :rtype: baidubce.bce_response.BceResponse
        """
        if name == '':
            raise BceClientError('name can\'t be empty string')
        return self._send_request(http_methods.DELETE, '/notification/' + name, config=config)

    def _get_config_parameter(self, config, attr):
        result = None
        if config is not None:
            result = getattr(config, attr)
        if result is not None:
            return result
        return getattr(self.config, attr)

    @staticmethod
    def _merge_config(self, config):
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(
            self, http_method, path,
            body=None, headers=None, params=None,
            config=None,
            body_parser=None):
        config = self._merge_config(self, config)
        if body_parser is None:
            body_parser = handler.parse_json
        realpath = MediaClient.prefix + path
        return bce_http_client.send_request(
            config, bce_v1_signer.sign, [handler.parse_error, body_parser],
            http_method, str.encode(realpath), body, headers, params)
